/*
 * Xelfi
 */

package xelfi.awt;

import java.awt.*;

/**
 * A class that produces a TrackBar.
 *
 * @version 2.1 1 Mar 1997
 * @author  Petr Hamernik
 */
public class TrackBar extends Canvas {

	/** When TrackBar position increases, parent will get 
	 *  the event with this id. 
	 */
    public static final int TB_LINEUP	= 751020;

	/** When TrackBar position decreases, parent will get 
	 *  the event with this id. 
	 */
    public static final int TB_LINEDOWN	= 751021;

	/** When TrackBar position increases by pagesize, parent will get 
	 *  the event with this id. 
	 */
    public static final int TB_PAGEUP	= 751022;
	
	/** When TrackBar position decreases by pagesize, parent will get 
	 *  the event with this id. 
	 */
    public static final int TB_PAGEDOWN	= 751023;
	
	/** When TrackBar position goes home, parent will get 
	 *  the event with this id. 
	 */
    public static final int TB_HOME		= 751024;
	
	/** When TrackBar position goes to the end, parent will get 
	 *  the event with this id. 
	 */
    public static final int TB_END		= 751025;
	
	/** When TrackBar is in dragging mode, parent will get 
	 *  the event with this id. 
	 */
	public static final int TB_DRAG		= 751026;

	/** Constant for setting the orientation of TrackBar.
	 * @see #orientation
	 * @see #DIR_VERTICAL
	 */
	public final static boolean DIR_HORIZONTAL = true;

	/** Constant for setting the orientation of TrackBar.
	 * @see #orientation
	 * @see #DIR_HORIZONTAL
	 */
	public final static boolean DIR_VERTICAL = false;

	/** Current orientation of TrackBar. Available values are
	 *  DIR_XXX constants.
	 * @see #defaultOrientation
	 * @see #setOrientation
	 * @see #getOrientation
	 * @see #DIR_HORIZONTAL
	 * @see #DIR_VERTICAL
	 */
    protected boolean orientation;

	/** Default orientation of TrackBar.
	 * @see #orientation
	 * @see #DIR_HORIZONTAL
	 */
	public static final boolean defaultOrientation = DIR_HORIZONTAL;

	/** Minimum of the range of the TrackBar.
	 * @see #defaultMinimum
	 * @see #setMinimum
	 * @see #getMinimum
	 */
    protected int minimum;

	/** Maximum of the range of the TrackBar.
	 * @see #defaultMaximum
	 * @see #setMaximum
	 * @see #getMaximum
	 */
    protected int maximum;

	/** Page step of the TrackBar.
	 * @see #defaultPageStep
	 * @see #setPageStep
	 * @see #getPageStep
	 */
	protected int pageStep;

	/** Line step of the TrackBar.
	 * @see #defaultLineStep
	 * @see #setLineStep
	 * @see #getLineStep
	 */
	protected int lineStep;

	/** Value of the TrackBar.
	 * @see #defaultValue
	 * @see #setValue
	 * @see #getValue
	 */
	protected int value;

	/** Default minimum. Currently 0.
	 * @see #minimum
	 * @see #setMinimum
	 * @see #getMinimum
	 */
    public static final int defaultMinimum = 0;

	/** Default maximum. Currently 10.
	 * @see #maximum
	 * @see #setMaximum
	 * @see #getMaximum
	 */
    public static final int defaultMaximum = 10;

	/** Default page step. Currently 2.
	 * @see #pageStep
	 * @see #setPageStep
	 * @see #getPageStep
	 */
    public static final int defaultPageStep = 2;

	/** Default line step. Currently 1.
	 * @see #lineStep
	 * @see #setLineStep
	 * @see #getLineStep
	 */
    public static final int defaultLineStep = 1;

	/** Default value. Currently 0.
	 * @see #value
	 * @see #setValue
	 * @see #getValue
	 */
	public static final int defaultValue = 0;

	/** Freqency of tick marks. Each {frequency} value will
	 *  be visible.
	 * @see #defaultFrequency
	 * @see #setFrequency
	 * @see #getFrequency
	 */
    protected int frequency;

	/** Default freqency of tick marks. Currently 1.
	 * @see #frequency
	 * @see #setFrequency
	 * @see #getFrequency
	 */
    public static final int defaultFrequency = 1;

	/** Tick marks are at the bottom or at the right side.
	 * @see #tickMarks
	 */
    public static final int TM_BOTTOMRIGHT = 0;
	
	/** Tick marks are at the top or at the left side.
	 * @see #tickMarks
	 */
    public static final int TM_TOPLEFT = 1;
	
	/** Tick marks are at the both sides of TrackBar.
	 * @see #tickMarks
	 */
    public static final int TM_BOTH = 2;

	/** Tick marks are not visible.
	 * @see #tickMarks
	 */
    public static final int TM_NONE = 3;

	/** Determine, whether the tick marks are visible.
	 *  Allowed values are the constants TM_XXX
	 * @see #TM_BOTTOMRIGHT
	 * @see #TM_TOPLEFT
	 * @see #TM_BOTH
	 * @see #TM_NONE
	 * @see #defaultTickMarks
	 * @see #setTickMarks
	 * @see #getTickMarks
	 */
	protected int tickMarks;

	/** Default value of tickMarks
	 * @see #TM_BOTTOMRIGHT
	 * @see #tickMarks
	 */
	public static final int defaultTickMarks = TM_BOTTOMRIGHT;

	/** Width of the lath of TrackBar.
	 * @see #defaultLathWidth
	 * @see #setLathWidth
	 * @see #getLathWidth
	 */
	protected int lathWidth;

	/** Default freqency of tick marks. Currently 8.
	 * @see #lathWidth
	 * @see #setLathWidth
	 * @see #getLathWidth
	 */
	public static final int defaultLathWidth = 8;

	/** Lenght of the tick marks.
	 * @see #defaultTickLenght
	 * @see #setTickLenght
	 * @see #getTickLenght
	 */
	protected int tickLenght;
	
	/** Default lenght of the tick marks. Currently 3.
	 * @see #tickLenght
	 * @see #setTickLenght
	 * @see #getTickLenght
	 */
	public static final int defaultTickLenght = 3;

	/** The basic color of the 3D border. Left and top sides are 
	 *  lighter and right and bottom sides are darker then this 
	 *  color. By default it is set to defaultBorderColor.
	 * @see #setBorderColor
	 * @see #getBorderColor
	 * @see #defaultBorderColor
	 */
	protected Color marginColor = null;

	/** Default color of the border. Currently lightGray.
	 * @see #setBorderColor
	 * @see #getBorderColor
	 * @see #marginColor
	 */
    public static final Color defaultBorderColor = new Color(192, 192, 192);

	/** The color for filling the lath. 
	 *  By default it is set to defaultFillColor.
	 * @see #setFillColor
	 * @see #getFillColor
	 * @see #defaultFillColor
	 */
    protected Color fillColor = null;

	/** Default color of the area inside the lath. Currently white.
	 * @see #setFillColor
	 * @see #getFillColor
	 * @see #fillColor
	 */
    public static final Color defaultFillColor = Color.white;

	/** If propagate is set to true all events are propagate and don't
	 *  catch. This flag is used for xawt component during design time.
	 */
	public boolean propagate = false;

	private int startpos = 0;
	private int endpos = 0;
	private boolean dragging = false;

	/** Create new TrackBar with all default values. */
	public TrackBar() {
		setDefaults();
	}

	private void setDefaults() {
		orientation = defaultOrientation;
		minimum = defaultMinimum;
		maximum = defaultMaximum;
		pageStep = defaultPageStep;
		lineStep = defaultLineStep;
		value = defaultValue;
		frequency = defaultFrequency;
		tickMarks = defaultTickMarks;
		lathWidth = defaultLathWidth;
		tickLenght = defaultTickLenght;
		marginColor = defaultBorderColor;
		fillColor = defaultFillColor;
	}

	/** Sets the new orientation.
	 * @param aDir new value of orientation.
	 * @see #orientation
	 * @see #defaultOrientation
	 * @see #getOrientation
	 */
	public void setOrientation(boolean aDir) {
		if (orientation == aDir)
			return;
		orientation = aDir;
		repaint();
	}

	/** Gets the current orientation of TrackBar.
	 * @return value of orientation.
	 * @see #orientation
	 * @see #defaultOrientation
	 * @see #setOrientation
	 */
	public boolean getOrientation() {
		return orientation;
	}

	/** Set a new lath width.
	 * @param aLathWidth New lath width.
	 * @see #lathWidth
	 * @see #getLathWidth
	 */
    public void setLathWidth(int aLathWidth) {
		if ((aLathWidth == lathWidth) || (aLathWidth < 1))
			return;
        lathWidth = aLathWidth;
        repaint();
    }

	/** Gets the current lath width of TrackBar.
	 * @return width of the lath.
	 * @see #lathWidth
	 * @see #setLathWidth
	 */
    public int getLathWidth() {
        return(lathWidth);
    }

	/** Sets a minimum of the range of the TrackBar. If value
	 *  or maximum fall out of acceptable values they are adjusted.
	 * @param aMin New minimum.
	 * @see #getMinimum
	 */
	public void setMinimum(int aMin) {
		if (minimum == aMin)
			return;
		minimum = aMin;
		if (value < minimum)
			value = minimum;
		if (maximum < minimum)
			maximum = minimum;
        repaint();
	}

	/** Gets the current minimum of the range of TrackBar.
	 * @return Minimum.
	 * @see #setMinimum
	 */
	public int getMinimum() {
		return minimum;
	}
	
	/** Sets a maximum of the range of the TrackBar. If value
	 *  or minimum fall out of acceptable values they are adjusted.
	 * @param aMax New maximum.
	 * @see #getMinimum
	 */
	public void setMaximum(int aMax) {
		if (maximum == aMax)
			return;
		maximum = aMax;
		if (value > maximum)
			value = maximum;
		if (maximum < minimum)
			minimum = maximum;
        repaint();
	}

	/** Gets the current maximum of the range of TrackBar.
	 * @return Maximum.
	 * @see #setMaximum
	 */
	public int getMaximum() {
		return maximum;
	}

	/** Sets a new value of the TrackBar. If value is outside
	 *  the ranges it is set to nearest acceptable value.
	 * @param aValue New value.
	 * @see #getValue
	 */
	public void setValue(int aValue) {
		if (value == aValue)
			return;
		int oldvalue = value;
		value = aValue;
		if (value < minimum)
			value = minimum;
		if (value > maximum)
			value = maximum;
        repaintPositions(oldvalue);
	}

	/** Gets the current value of the TrackBar.
	 * @return Value.
	 * @see #setValue
	 */
	public int getValue() {
		return value;
	}

	/** Sets a new page step of the TrackBar.
	 * @param aStep New page step.
	 * @see #getPageStep
	 */
	public void setPageStep(int aStep) {
		pageStep = aStep;
	}

	/** Gets the current page step of the TrackBar.
	 * @return Page step.
	 * @see #setPageStep
	 */
	public int getPageStep() {
		return pageStep;
	}

	/** Sets a new line step of the TrackBar.
	 * @param aStep New line step.
	 * @see #getLineStep
	 */
	public void setLineStep(int aStep) {
		lineStep = aStep;
	}

	/** Gets the current line step of the TrackBar.
	 * @return Line step.
	 * @see #setLineStep
	 */
	public int getLineStep() {
		return lineStep;
	}

	/** Sets a new lenght of the tick marks.
	 * @param aTickLenght New lenght of the tick marks.
	 * @see #defaultTickLenght
	 * @see #tickLenght
	 * @see #getTickLenght
	 */
	public void setTickLenght(int aTickLenght) {
		if ((aTickLenght < 1) || (aTickLenght == tickLenght))
			return;
		tickLenght = aTickLenght;
        repaint();
	}

	/** Gets the current lenght of the tick marks.
	 * @return Lenght of the ticks.
	 * @see #defaultTickLenght
	 * @see #tickLenght
	 * @see #setTickLenght
	 */
	public int getTickLenght() {
		return tickLenght;
	}

	/** Set a new freqency of tick marks.
	 * @param aFrequency New freqency.
	 * @see #defaultFrequency
	 * @see #frequency
	 * @see #getFrequency
	 */
	public void setFrequency(int aFrequency) {
		if ((aFrequency < 1) || (aFrequency == frequency))
			return;
		frequency = aFrequency;
        repaint();
	}

	/** Set a new freqency of tick marks.
	 * @return Frequency.
	 * @see #defaultFrequency
	 * @see #frequency
	 * @see #getFrequency
	 */
	public int getFrequency() {
		return frequency;
	}

	/** Sets new value of tickMarks. Allowed values are 
	 * the constants TM_XXX.
	 * @param aTM new value of tickMarks
	 * @see #TM_BOTTOMRIGHT
	 * @see #TM_TOPLEFT
	 * @see #TM_BOTH
	 * @see #TM_NONE
	 * @see #getTickMarks
	 * @see #tickMarks
	 */
	public void setTickMarks(int aTM) throws ArrayIndexOutOfBoundsException {
		if (tickMarks == aTM)
			return;
		if ((aTM < 0) || (aTM > 3))
	        throw new ArrayIndexOutOfBoundsException();
		tickMarks = aTM;
        repaint();
	}

	/** Gets the value of tickMarks.
	 * @return Value of tickMarks.
	 * @see #TM_BOTTOMRIGHT
	 * @see #TM_TOPLEFT
	 * @see #TM_BOTH
	 * @see #TM_NONE
	 * @see #setTickMarks
	 * @see #tickMarks
	 */
	public int getTickMarks() {
		return tickMarks;
	}

	/** Sets the border color.
	 * @param col New value of marginColor.
	 * @see #marginColor
	 * @see #defaultBorderColor
	 * @see #getBorderColor
	 */
    public void setBorderColor(Color col) {
        marginColor = col;
        repaint();
    }

	/** Gets the border color.
	 * @return Current border color.
	 * @see #marginColor
	 * @see #defaultBorderColor
	 * @see #setBorderColor
	 */
    public Color getBorderColor() {
        return marginColor;
    }

	/** Sets the fill color.
	 * @param col New value of fillColor.
	 * @see #fillColor
	 * @see #defaultFillColor
	 * @see #getFillColor
	 */
    public void setFillColor(Color col) {
        fillColor = col;
        repaint();
    }

	/** Gets the fill color.
	 * @return Current fill color.
	 * @see #fillColor
	 * @see #defaultFillColor
	 * @see #setFillColor
	 */
    public Color getFillColor() {
        return fillColor;
    }

	public void paint(Graphics g) {
		Dimension d = size();

		int left = 0;
		int top = 0;
		int w = d.width - 1;
		int h = d.height - 1;

	    g.setColor(getBackground());
		g.fillRect(left,top,w+1,h+1);

		int x = 0;
		int y = 0;
		if (orientation == DIR_HORIZONTAL) {
			x = 4;
			y = 4;
			if ((tickMarks == TM_TOPLEFT) || (tickMarks == TM_BOTH))
				y += tickLenght - 1;
			w = w - 2*x - 1;
			h = lathWidth + 3;
			startpos = x + 4;
			endpos = x + w - 4;
		}
		else {
			y = 4;
			x = 4;
			if ((tickMarks == TM_TOPLEFT) || (tickMarks == TM_BOTH))
				x += tickLenght - 1;
			h = h - 2*y - 1;
			w = lathWidth + 3;
			startpos = y + 4;
			endpos = y + h - 4;
		}
		paintLath(g, x, y, w, h);
		
		if (tickMarks != TM_NONE) {
			if (orientation == DIR_HORIZONTAL) {
				x = startpos;
				w = endpos - startpos;
				y = 0;
				h = tickLenght;
			}
			else {
				y = startpos;
				h = endpos - startpos;
				x = 0;
				w = tickLenght;
			}
			if ((tickMarks == TM_TOPLEFT) || (tickMarks == TM_BOTH))
				paintTicks(g, x, y, w, h);
			if ((tickMarks == TM_BOTTOMRIGHT) || (tickMarks == TM_BOTH)) {
				int dd = 1;
				if (tickMarks == TM_BOTH)
					dd = tickLenght;
				if (orientation == DIR_HORIZONTAL)
					y = 10 + dd + lathWidth;
				else
					x = 10 + dd + lathWidth;
				paintTicks(g, x, y, w, h);
			}
		}
			
		paintCursor(g);
	}

	private void paintLath(Graphics g, int x, int y, int w, int h) {
		g.setColor(marginColor);
		g.draw3DRect(x, y, w, h, false);
		x++;
		y++;
		w -= 2;
		h -= 2;
		g.setColor(fillColor);
		g.fillRect(x,y,w,h);
	    g.setColor(Color.black);
	    g.drawLine(x, y, x + w, y);
	    g.drawLine(x, y, x, y + h);
	}

	private void paintTicks(Graphics g, int x, int y, int w, int h) {
		g.setColor(getForeground());
		int total = maximum - minimum;
		int dif = h;
		if (orientation == DIR_HORIZONTAL)
			dif = w;
		
		for (int i = 0; i <= total; i++) {
			if ((i % frequency != 0) && ( i != total))
				continue;
			int delta = i * dif / total;
			if (orientation == DIR_HORIZONTAL)
				g.drawLine(x + delta, y, x + delta, y + tickLenght -1);
			else
				g.drawLine(x, y + delta, x + tickLenght - 1, y + delta);
		}
	}

	private void paintCursor(Graphics g) {
		Point p = getCursorPos();
		Dimension d = getCursorDim();

		g.setColor(getBackground());
		g.fillRect(p.x, p.y, d.width, d.height);

		g.setColor(marginColor);
		g.draw3DRect(p.x, p.y, d.width - 2, d.height - 2, true);

	    g.setColor(Color.black);
	    g.drawLine(p.x, p.y + d.height - 1, p.x + d.width - 1, p.y + d.height - 1);
	    g.drawLine(p.x + d.width - 1, p.y, p.x + d.width - 1, p.y + d.height - 1);
	}

	private void repaintPositions(int oldpos) {
		Dimension d = getCursorDim();

		int xxx = value;
		value = oldpos;
		Point p = getCursorPos();
		value = xxx;
		repaint(p.x,p.y, d.width, d.height);				

		p = getCursorPos();
		repaint(p.x,p.y, d.width, d.height);				
	}

	private Dimension getCursorDim() {
		int w = 9;
		int h = 9;
		if (orientation == DIR_HORIZONTAL)
			h = lathWidth + 8;
		else
			w = lathWidth + 8;
		return new Dimension(w,h);
	}

	private Point getCursorPos() {
		int d1 = startpos + ((value - minimum) * (endpos - startpos)) / (maximum - minimum) - 4;
		int d2 = 2;
		if ((tickMarks == TM_TOPLEFT) || (tickMarks == TM_BOTH))
			d2 += tickLenght - 1;
		if (orientation == DIR_HORIZONTAL)
			return new Point(d1, d2);
		else
			return new Point(d2, d1);
	}

	private int resolveClick(int x, int y) {
		Point p = getCursorPos();
		Dimension d = getCursorDim();
		if ((x >= p.x) && (x < p.x + d.width) && (y >= p.y) && (y < p.y + d.height))
			return 0;
		if (orientation == DIR_HORIZONTAL) {
			if (x < p.x)
				return -1;
			if (x >= p.x + d.width)
				return 1;
			return 99;
		}
		else {
			if (y < p.y)
				return -1;
			if (y >= p.y + d.height)
				return 1;
			return 99;
		}
	}

	private int resolveValue(int x, int y) {
		int v = 0;
		int dif = (endpos - startpos) / (2*(maximum - minimum));

		if (orientation == DIR_HORIZONTAL) {
			if (x < startpos)
				v = minimum;
			else if (x > endpos)
				v = maximum;
			else v = ((x + dif - startpos) * (maximum - minimum)) / (endpos - startpos);
		}
		else {
			if (y < startpos)
				v = minimum;
			else if (y > endpos)
				v = maximum;
			else v = ((y + dif - startpos) * (maximum - minimum)) / (endpos - startpos);
		}
		return v;
	}

    public boolean mouseDown(Event event, int x, int y) {
		if (propagate)
			return false;

		int oldvalue = value;
		dragging = false;
		switch (resolveClick(x,y)) {
			case -1:value -= pageStep;
					event.id = TB_PAGEUP;
					if (value <= minimum)
						value = minimum;
					break;
			case 1 :value += pageStep;
					event.id = TB_PAGEDOWN;
					if (value >= maximum)
						value = maximum;
					break;
			case 0: dragging = true;
					break;
		}
		if (value != oldvalue) {
			repaintPositions(oldvalue);
			return false;
		}
		return true;
	}
	
    public boolean mouseUp(Event event, int x, int y) {
		if (propagate)
			return false;
		if (dragging) {
			int oldvalue = value;
			value = resolveValue(x,y);
			if (value != oldvalue) {
				event.id = TB_DRAG;
				repaintPositions(oldvalue);
				dragging = false;
				return false;
			}
		}
		return true;
	}
	
	public boolean mouseDrag(Event evt, int x, int y) {
		if (propagate || !dragging)
			return false;
		int oldvalue = value;
		value = resolveValue(x,y);
		if (value != oldvalue) {
			evt.id = TB_DRAG;
			repaintPositions(oldvalue);
			return false;
		}
		return true;
	}

    public boolean keyDown(Event event, int key) {
		if (propagate)
			return false;
		int oldvalue = value;
		if ((key == Event.RIGHT) || (key == Event.DOWN)) {
			value += lineStep;
			event.id = TB_LINEDOWN;
			if (value >= maximum)
				value = maximum;
    	}
		else if ((key == Event.LEFT) || (key == Event.UP)) {
			value -= lineStep;
			event.id = TB_LINEUP;
			if (value <= minimum)
				value = minimum;
    	}
		else if ((key == Event.PGDN)) {
			value += pageStep;
			event.id = TB_PAGEDOWN;
			if (value >= maximum)
				value = maximum;
    	}
		else if ((key == Event.PGUP)) {
			value -= pageStep;
			event.id = TB_PAGEUP;
			if (value <= minimum)
				value = minimum;
    	}
		else if ((key == Event.END)) {
			value = maximum;
			event.id = TB_END;
    	}
		else if ((key == Event.HOME)) {
			value = minimum;
			event.id = TB_HOME;
    	}
		if (value != oldvalue) {
			repaintPositions(oldvalue);
			return false;
		}
		return true;
	}

    public boolean handleEvent(Event event) {
		if (propagate)
			return false;
    	return super.handleEvent(event);
	}	

    public Dimension minimumSize() {
        return countSize(true);
    }

    public Dimension preferredSize() {
        return countSize(false);
    }

	private Dimension countSize(boolean Min) {
		int a = (maximum - minimum + 1) / frequency;
		if (Min) 
			a = a * 2;
		else
			a = a * 10;
		a += 17;
		int b = lathWidth + 10;
		switch (tickMarks) {
			case TM_BOTTOMRIGHT:
			case TM_TOPLEFT: b += tickLenght + 1; break;
			case TM_BOTH: b += 2 * tickLenght; break;
			case TM_NONE: b += 2; break;
		}
		if (orientation == DIR_HORIZONTAL)
			return new Dimension(a,b);
		else
			return new Dimension(b,a);
	}
}
