/**
* Xelfi
*/

package xelfi.design.components.dialogs;

import java.awt.*;
import xelfi.design.components.*;
import xelfi.design.util.XFDesignConstants;
import xelfi.design.components.property.XFImageName;
import xelfi.top.TopLevel;

/**
 * FontSelectThread class is here only to get working modal XFFontDialog 
 * with JDK 1.02 under Win32 (Sunsoft claimed that this bug is fixed in JDK 1.02
 * but not under Win32 - the show() method of the Dialog class is still not blocking)
 *
 * @version 	1.2, 16 Dec 1996
 * @author 	Ian Formanek, Petr Hamernik
 */

public class ImageNameSelectThread extends Thread {
	private Frame parent;
	private XelfiDesignComponent xdc;
	private int propNum;
	private Container compParent;
	
	/**
	 * Constructor to set parameters for the thread and for the font dialog
	 * @param	name				the name of the thread
	 * @param	aParent			parent of the dialog
	 * @param	aXdc				Xelfi component whose Font propety to set
	 * @param	aPropNum		index of the Font property
	 * @param	aCompParent	parent of the component which validate() method is called 
	 */
	public ImageNameSelectThread(String name, Frame aParent, 
			XelfiDesignComponent aXdc, int aPropNum) {
		super(name);		
		parent = aParent;
		xdc = aXdc;
		propNum = aPropNum;
	}

	/**
	 * A method which actually runs the XFFontDialog to select fonts.
	 * It first gets current font settings from the component and then invokes the 
	 * XFFontDialog. Since the method show() immediately returns, it starts an "infinite"
	 * loop which checks whether the dialog has finished. Then, if user clicked OK, the 
	 * font property of the component is set according to the font selected in the dialog
	 * and the component is repainted (by calling its invalidate() and validate() methods).
	 */
	public void run() {
		Object of = null;
		try {
			of = xdc.getProperty(propNum);
			} catch (XFPropertyIndexOutOfBoundsException e) {
		}
		if (of instanceof XFImageName) {
			XFImageNameDialog fsd = new XFImageNameDialog(parent);
			fsd.setData((XFImageName) of);
			fsd.show();
			// for the dialog to run at reasonable speed
			// we must set to this loop minimal priority
			setPriority(MIN_PRIORITY);
			do { } while (fsd.finished != true);
			
			if (fsd.result == XFFontDialog.RES_OK) {
	
				// set the font property and catch exceptions
				try {
					int ret = xdc.setProperty(propNum, fsd.getData());
					if ((ret & XFDesignConstants.RSP_OTHER_PROP) != 0)
						TopLevel.getInspector().propertiesChanged();
					} catch (XFPropertyBadValueTypeException e) {	
					} catch (XFPropertyIndexOutOfBoundsException e) {
					}
				}
			}
		else System.out.println("Property number: "+ propNum + " of component: "+ xdc +" is not of type ImageFileName");
		}
}