/**
* Xelfi
	Last change:  DF   16 Mar 97    6:18 pm
*/

package xelfi.design.components.xawt;

import java.awt.*;
import java.io.*;

import xelfi.design.components.*;
import xelfi.design.components.property.*;
import xelfi.design.util.XFDesignConstants;
import xelfi.design.forms.XelfiDesignForm;
import xelfi.top.TopLevel;
import xelfi.design.inspector.XelfiMenuEditor;
import xelfi.util.Debug;


/**
 * Xelfi design component for menu folders.
 * 
 * @version		1.11, 25 Feb 1997
 * @author		Dafe, Ian Formanek
*/
public class XMenuFolder extends Menu implements XelfiDesignComponent {

  private static int        propertyCount = 2;
	private String				    name = "folder";
	private boolean				    propDefSet = false;
  private XelfiMenuEditor   me;
  // parent form reference
  private XelfiDesignForm       parentForm;

	public XMenuFolder () {
		this("");
	}

	public XMenuFolder (String caption) {
		super(caption);
    // get the ref. to the menu editor
    me = getMenuEditor();
	}

	public void setParentForm(XelfiDesignForm form) {
    parentForm = form;
  }

  public XelfiDesignForm getParentForm () {
    return parentForm;
  }

	public int getPropertyCount () { return propertyCount; }
	
	public Object getProperty (int index) 
				throws XFPropertyIndexOutOfBoundsException { 
		switch (index) {
			case 0: 
				return getLabel();
			case 1:
				return isEnabled() ? Boolean.TRUE : Boolean.FALSE;
/*      case 1:
        return getFont(); */
			default: 
				throw new XFPropertyIndexOutOfBoundsException();
		}
	}
			
	public String[] getPropertyNames () { 
		String list[] = new String[propertyCount];
		list[0] = new String("Caption"); 
		list[1] = new String("Enabled");
/*    list[1] = new String("Font"); */
		return list;
	}
		
	public String getPropertyString (int index)
		throws XFPropertyIndexOutOfBoundsException { 
		switch (index) {
			case 0: 
				return XStringPropertyType.viewString(getProperty(index));
			case 1:
				return XBooleanPropertyType.viewString(getProperty(index));
/*      case 1:
        return XFontPropertyType.viewString(getProperty(index)); */
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}

	public int getPropertyViewerType (int index) 
		throws XFPropertyIndexOutOfBoundsException {
		switch (index) {
			case 0: 
				return XFDesignConstants.PVT_INPUT_STRING;
			case 1:
				return XFDesignConstants.PVT_CHECK;
/*      case 2:
        return XFDesignConstants.PVT_SPECIAL; */
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}

	public String[] getPropertyValueNames (int index)
		throws XFPropertyIndexOutOfBoundsException {
		switch (index) {
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}

	public Object[] getPropertyValueValues (int index)
		throws XFPropertyIndexOutOfBoundsException {
		switch (index) {
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}

	public String[] getPropertyValuesArray (int index) 
		throws XFPropertyIndexOutOfBoundsException {
		switch (index) {
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}

	public int setProperty (int index, Object value) 
		throws XFPropertyIndexOutOfBoundsException,
						XFPropertyBadValueTypeException 
	{ 
		int ret = XFDesignConstants.RSP_OK;
		int regen = XFDesignConstants.GEN_SETPROPS;

    // menu must be removed before any editing action
    if ( me!=null ) me.removeMenu();

		if (!propDefSet)
			propertiesSetDefault();

		switch (index) {
			case 0: 
				if (!(value instanceof String)) 
						throw new XFPropertyBadValueTypeException();
        regen = regen | XFDesignConstants.GEN_INIT;
 		    setLabel((String)value);
				break;
			case 1:
				if (!(value instanceof Boolean)) 
						throw new XFPropertyBadValueTypeException();
				enable(((Boolean)value).booleanValue());
				break;
/*      case 2:
        if (!(value instanceof Font))
          throw new XFPropertyBadValueTypeException();
        setFont((Font)value);
        ret = XFDesignConstants.RSP_REPAINT_COMP;
        break; */
			default:
				throw new XFPropertyIndexOutOfBoundsException();
		}

    // regenerate source code
    regenForm(regen);
    
    // and here we come...setting the menu again
		// and refresh me's content too
    if (me!=null) {
			me.refresh();
		  me.setMenu();
		}
		return ret;
	}
			
	public void specialPropertyInput (Frame parent, int index) 
		throws XFPropertyIndexOutOfBoundsException, 
						XFPropertyNoSpecialInputException { 
		if (index == 0) {
			// [PENDING] - open menu editor window
			// probably will call top-window method
		}
		else throw new XFPropertyIndexOutOfBoundsException();
	}

	public int getEventCount() {
		return 0;
	}

	public String[] getEventNames() {
		return null;
	}

	public String getEvent(int index)
		throws XFPropertyIndexOutOfBoundsException {
		switch (index) {
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}		
		
	public void setEvent(int index, String value)
		throws XFPropertyIndexOutOfBoundsException {
		switch (index) {
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}		

	public String getName () { 
		return name;
	}

	/**
	 * Sets the name of the component.
	 * Refreshes menu editor if needed.
	*/
	public int setDesignName (String name) {
		this.name = name;
		return XFDesignConstants.RSP_OK;
	}

	public Image getIcon () {
		return null; 
	}
	
	public String getClassName () { return "Menu"; }
	
	public String generateEventCondition(int index) throws XFPropertyIndexOutOfBoundsException {
		throw new XFPropertyIndexOutOfBoundsException();
	}

	public String[] neededImports() {
		String list[] = new String[1];
		list[0] = "java.awt.Menu";
		return list;
	}

	public String generateCode (int what) { 
		String gen = "";
		switch (what) {
			case XFDesignConstants.GEN_VARIABLE: 
				gen = getClassName()+" "+ name + ";\n";
				for (int i=0; i<countItems(); i++)
					gen = gen+((XelfiDesignComponent)getItem(i)).generateCode(XFDesignConstants.GEN_VARIABLE);
				break;
			case XFDesignConstants.GEN_INIT: 
				gen = name + " = new " + getClassName() + "(\""+getLabel()+"\");\n";
				gen = gen+((XelfiDesignComponent)getParent()).getName()+".add("+name+");\n";
				for (int i=0; i<countItems(); i++)
					gen = gen+((XelfiDesignComponent)getItem(i)).generateCode(XFDesignConstants.GEN_INIT);
				break;
		}
		return gen;
	}

	public void loadFromStream (DataInput stream) 
		throws IOException {
		name = stream.readUTF();		
		parentForm.setComponentName(this, name);
		setLabel(stream.readUTF());
//		setFont(XFontPropertyType.loadFromStream(stream));
		enable(XBooleanPropertyType.loadBFromStream(stream));
		// create temporary objects (getClassName is not static yet :-(
		XMenuFolder	tempMenuFolder = new XMenuFolder();
		XMenuItem	tempMenuItem = new XMenuItem();
		XCheckboxMenuItem	tempCheckboxMenuItem = new XCheckboxMenuItem();
		int count = stream.readInt();
		for (int i=0; i<count; i++) {
			// read class info (name) and decide 
			// to which type our current child belongs
			String classInfo = stream.readUTF();
			if (classInfo.equals(tempMenuItem.getClassName())) {
				XMenuItem xmn = new XMenuItem();
				xmn.setParentForm(parentForm);
				xmn.loadFromStream(stream);
				add(xmn);
			}
			else {
			if (classInfo.equals(tempMenuFolder.getClassName())) {
				XMenuFolder xmf = new XMenuFolder();
				xmf.setParentForm(parentForm);
				xmf.loadFromStream(stream);
				add(xmf);
			}
			else {
			if (classInfo.equals(tempCheckboxMenuItem.getClassName())) {
				XCheckboxMenuItem xmn = new XCheckboxMenuItem();
				xmn.setParentForm(parentForm);
				xmn.loadFromStream(stream);
				add(xmn);
			}}}
		}
	}
	
	public void saveToStream (DataOutput stream)
		throws IOException {
		stream.writeUTF(name);
		stream.writeUTF(getLabel());
	//	XFontPropertyType.saveToStream(getFont(), stream);
		XBooleanPropertyType.saveToStream(isEnabled(), stream);
		int count = countItems();
		stream.writeInt(count);
		for (int i=0; i<count; i++) {
			MenuItem mni = getItem(i);
			// write class info (name)
			stream.writeUTF(((XelfiDesignComponent)mni).getClassName());
			if (mni instanceof XMenuItem) {
				((XMenuItem)mni).saveToStream(stream);
			}
			else {
			if (mni instanceof XMenuFolder) {
				((XMenuFolder)mni).saveToStream(stream);
			}
			else {
			if (mni instanceof XCheckboxMenuItem) {
				((XCheckboxMenuItem)mni).saveToStream(stream);
			}}}
		}
	}

	public boolean handleEvent (Event evt) {
		return false;
	} 
	
	public boolean parentUpdate () {
		return false;
	}

	private void propertiesSetDefault () {
	}

	public void setSelection(int aSelection) {
	}

	public int getSelection() {
		return XFDesignConstants.SEL_UNSELECT;
	}

/**** private *****/
  
  private void regenForm(int what) {
    if (parentForm != null) {
      parentForm.regenerateCode(this, what);
      Debug.debug(Debug.MENU, "regenerating source in the FORM!!!!");
    }
  }

  /**
   * Returns refernce to menu editor object.
   * (I'm asking Filip's top level object for it)
  */
  private XelfiMenuEditor getMenuEditor () {
    return TopLevel.getMenuEditor();
  }

	/**
	 * Refreshes content of menu editor.
	*/
	private void refreshME () {
    // get the reference to ME
    XelfiMenuEditor me = getMenuEditor();
    if (me==null) return;
		me.refresh();
	}


}
