/**
* Xelfi
*/

package xelfi.design.components.xawt;

import java.awt.*;
import java.io.*;

import xelfi.design.components.*;
import xelfi.design.components.property.*;
import xelfi.design.util.XFDesignConstants;
import xelfi.design.util.XFDesignOptions;
import xelfi.design.forms.XelfiDesignForm;
import xelfi.top.TopLevel;
import xelfi.design.inspector.XelfiMenuEditor;
import xelfi.util.Debug;

/**
 * Xelfi design component for non-checkable menu items
 * 
 * @version		1.11, 25 Feb 1997
 * @author		Dafe, Ian Formanek
*/
public class XMenuItem extends MenuItem implements XelfiDesignComponent {

	private static int propertyCount = 2;
	private static int eventCount = 1;
	private String events[] = new String[eventCount];

	private String name;
	private XelfiMenuEditor me;
  // parent form reference
  private XelfiDesignForm parentForm;

	public XMenuItem () {
		this("");
	}

	public XMenuItem (String caption) {
		super(caption);
    me = getMenuEditor();
	}

	public void setParentForm(XelfiDesignForm form) {
    parentForm = form;
  }

  public XelfiDesignForm getParentForm () {
    return parentForm;
  }

	public int getPropertyCount () { return propertyCount; }
	
	public Object getProperty (int index) 
				throws XFPropertyIndexOutOfBoundsException { 
		switch (index) {
			case 0: 
				return getLabel();
/*			case 1:
				return getFont(); */
			case 1:
				return isEnabled() ? Boolean.TRUE : Boolean.FALSE;
			default: 
				throw new XFPropertyIndexOutOfBoundsException();
		}
	}
			
	public String[] getPropertyNames () { 
		String list[] = new String[propertyCount];
		list[0] = new String("Caption"); 
/*		list[1] = new String("Font"); */
		list[1] = new String("Enabled");
		return list;
	}
		
	public String getPropertyString (int index)
		throws XFPropertyIndexOutOfBoundsException { 
		switch (index) {
			case 0: 
				return XStringPropertyType.viewString(getProperty(index));
/*			case 1:
				return XFontPropertyType.viewString(getProperty(index)); */
			case 1:
				return XBooleanPropertyType.viewString(getProperty(index));
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}

	public int getPropertyViewerType (int index) 
		throws XFPropertyIndexOutOfBoundsException {
		switch (index) {
			case 0: 
				return XFDesignConstants.PVT_INPUT_STRING;
/*			case 1:
				return XFDesignConstants.PVT_SPECIAL; */
			case 1:
				return XFDesignConstants.PVT_CHECK;
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}

	public String[] getPropertyValueNames (int index)
		throws XFPropertyIndexOutOfBoundsException {
		switch (index) {
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}

	public Object[] getPropertyValueValues (int index)
		throws XFPropertyIndexOutOfBoundsException {
		switch (index) {
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}

	public String[] getPropertyValuesArray (int index) 
		throws XFPropertyIndexOutOfBoundsException {
		switch (index) {
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}

	public int setProperty (int index, Object value) 
		throws XFPropertyIndexOutOfBoundsException,
						XFPropertyBadValueTypeException 
	{ 
		int ret = XFDesignConstants.RSP_OK;
		int regen = XFDesignConstants.GEN_SETPROPS;

    // menu must be removed before any editing action
    if ( me!=null ) me.removeMenu();

		switch (index) {
			case 0: 
				if (!(value instanceof String)) 
						throw new XFPropertyBadValueTypeException();
        regen = regen | XFDesignConstants.GEN_INIT;
        setLabel((String)value);
				break;
/*			case 1:
				if (!(value instanceof Font)) 
						throw new XFPropertyBadValueTypeException();
				setFont((Font)value); 
				ret = XFDesignConstants.RSP_REPAINT_COMP;
				break; */
			case 1:
				if (!(value instanceof Boolean)) 
						throw new XFPropertyBadValueTypeException();
				enable(((Boolean)value).booleanValue());
				break;
			default:
				throw new XFPropertyIndexOutOfBoundsException();
		}
    // regenerate source code
    regenForm(regen);
    
    // and here we come...setting the menu again
    if ( me!=null ) {
		  me.refresh();
			me.setMenu();
		}
		return ret;
	}
			
	public void specialPropertyInput (Frame parent, int index) 
		throws XFPropertyIndexOutOfBoundsException, 
						XFPropertyNoSpecialInputException { 
		throw new XFPropertyIndexOutOfBoundsException();
	}

	public int getEventCount() {
		return eventCount;
	}

	public String[] getEventNames() {
		String list[] = new String[eventCount];
		list[0] = new String("OnSelect");
		return list;
	}

	public String getEvent(int index)
		throws XFPropertyIndexOutOfBoundsException {
		if ((index>=0) && (index<eventCount)) {
			if (events[index] == null)
				return null;
			else
				return new String(events[index]); 
		}
		else throw new XFPropertyIndexOutOfBoundsException();
	}		
		
	public void setEvent(int index, String value)
		throws XFPropertyIndexOutOfBoundsException {
		if ((index>=0) && (index<eventCount))
			events[index] = value;
		else throw new XFPropertyIndexOutOfBoundsException();
	}		

	public String getName () { 
		return name;
	}

	public int setDesignName (String aName) { 
		name = aName;
		return XFDesignConstants.RSP_OK;
	}

	public Image getIcon () {
		return null; 
	}
	
	public String getClassName () { return "MenuItem"; }
	
	public String[] neededImports() {
		String list[] = new String[0];
		return list;
	}

	public String generateEventCondition(int index) throws XFPropertyIndexOutOfBoundsException {
		String evt = XFDesignOptions.eventVarName;
		switch (index) { 
			case 0: return evt +".id == Event.ACTION_EVENT && "+evt+".target == "+name;
			default: throw new XFPropertyIndexOutOfBoundsException();
		}
	}

	public String generateCode (int what) { 
		String gen = "";
		switch (what) {
			case XFDesignConstants.GEN_VARIABLE: 
				gen = getClassName()+" "+ name + ";\n";
				break;
			case XFDesignConstants.GEN_INIT: 
				gen = name + " = new " + getClassName() + "(\""+getLabel()+"\");\n";
				gen = gen+((XelfiDesignComponent)getParent()).getName()+".add("+name+");\n";
				break;
		}
		return gen;
	}

	public void loadFromStream (DataInput stream) 
		throws IOException {
		name = stream.readUTF();		
		parentForm.setComponentName(this, name);
		setLabel(stream.readUTF());
//		setFont(XFontPropertyType.loadFromStream(stream));
		enable(XBooleanPropertyType.loadBFromStream(stream));

		for (int i = 0; i < eventCount; i++) {
			if (stream.readBoolean()) {
				Debug.debug(Debug.FORMIO_LOAD, "XMenuItem: event ["+i+"] loading...");
				parentForm.assignEvent(this, i, stream.readUTF());
			}
		}
	}
	
	public void saveToStream (DataOutput stream)
		throws IOException {
		stream.writeUTF(name);
		stream.writeUTF(getLabel());
//		XFontPropertyType.saveToStream(getFont(), stream);
		XBooleanPropertyType.saveToStream(isEnabled(), stream);

		for (int i = 0; i < eventCount; i++) {
			if ((events[i] != null) && (events[i].length() > 0)) {
				stream.writeBoolean(true);
				stream.writeUTF(events[i]);
			}
			else stream.writeBoolean(false);
		}
	}

	public boolean handleEvent (Event evt) {
		return false;
	} 
	
	public boolean parentUpdate () {
		return false;
	}

	public void setSelection(int aSelection) {
	}

	public int getSelection() {
		return XFDesignConstants.SEL_UNSELECT;
	}

  /**** private *****/
  
  private void regenForm(int what) {
    if (parentForm != null) {
      parentForm.regenerateCode(this, what);
      Debug.debug(Debug.MENU, "regenerating source in the FORM!!!!");
    }
  }
  
  /**
   * Returns refernce to menu editor object.
   * (I'm asking Filip's top level object for it)
  */
  private XelfiMenuEditor getMenuEditor () {
    return TopLevel.getMenuEditor();
  }

	/**
	 * Refreshes content of menu editor.
	*/
	private void refreshME () {
    // get the reference to ME
    XelfiMenuEditor me = getMenuEditor();
    if (me==null) return;
		me.refresh();
	}

}
