/**
* Xelfi programming environment
*/

package xelfi.editor;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.NoSuchElementException;
import xelfi.util.DuplicateException;
/**
* Implements methods of ActionController to call specified method on a block
* that contains relevate position.
*
* @see Block
*
* @version 1.0
* @author Jaroslav Tulach
*/

public class BlockController extends Object implements ActionController {
  /** table to store whole blocks and its controlers */
  private Hashtable table = new Hashtable ();

  /** block container to easily find the right block */
  private BlockContainer blocks = new BlockContainer ();

  /**
  * Adds new block and its controler to this object.
  * @param b the block to add
  * @param ac its controler
  * @exception DuplicateException if these blocks overlaps
  */
  public void add (Block b, ActionController ac) throws DuplicateException {
    blocks.add (b);
    table.put (b, ac);
  };

  /**
  * Removes block b from this object.
  * @param b the block to remove
  */
  public void remove (Block b) {
    try {
      blocks.remove (b);
    } catch (NoSuchElementException ex) {
    };
    table.remove (b);
  };

  /** This method is called before every insert to any position
  * For multiple inserts (of arrays or Strings) is called only
  * once at begining.
  *
  *
  * @param p the position method insert will insert to
  * @param info the info about delete
  * @exception NotAllowedException if this insert is not allowed
  */
  public void allowInsert (Position p, Object info) throws NotAllowedException {
    try {
      Block b = blocks.find (p, Block.INSIDE | Block.END);
      ((ActionController)table.get (b)).allowInsert (p, info);
    } catch (NoSuchElementException ex) {
    };
  };

  /** This method is called before every delete at any position
  * It is called for every position that the delete will remove.
  *
  * @param p the position method delete will delete at
  * @param info the info about delete
  * @exception NotAllowedException if this delete is not allowed
  */
  public void allowDelete (Position p, Object info) throws NotAllowedException {
    try {
      Block b = blocks.find (p, Block.INSIDE | Block.BEGIN);
      ((ActionController)table.get (b)).allowDelete (p, info);
    } catch (NoSuchElementException ex) {
    };
  };

  /** This method is called before every multi delete at any position
  *
  * @param p the position method delete will delete at
  * @exception NotAllowedException if this delete is not allowed
  */
  public void allowMultiDelete (Position beg, Position end, Object info)
  throws NotAllowedException {
    Enumeration e = table.keys ();
    
    try {
      for (;;) {
        Block b = (Block)e.nextElement ();
        if (beg == null || end == null || (
          b.getBegin ().lessOrEqual (end) && b.getEnd ().greater (beg)
        )) {
          ((ActionController)table.get (b)).allowMultiDelete (beg, end, info);
        }
      }
    } catch (NoSuchElementException ex) {
    }
  }
}


   
