/**
* Xelfi programming environment
*/

package xelfi.editor;
import java.awt.*;
/**
* Basic editor callback.
*
* @version 1.0
* @author Jaroslav Tulach
*/

class EditorInfo extends Object implements EditorCallback {
  Font f = new Font ("Courier", Font.PLAIN, 14);
  FontMetrics fm = Toolkit.getDefaultToolkit ().getFontMetrics (f);
  Dimension dim = new Dimension (fm.charWidth ('W'), fm.getHeight ());

  /** Returns the height and width of every character used.
  * @return the height
  */
  public Dimension getCharDimension () {
    return dim;
  };
  
  /** Returns the maximum ascent of any character used in document
  * @return the max ascent in points
  */
  public int getMaxAscent () {
    return fm.getMaxAscent ();
  };

  /** This method is called when a text has to be redrawn from
  * position p.
  * @param p the position of text that should be redrawn
  * @return normal TextDrawInfo extended with the information
  * in the field first that indicates the first position where
  * the text drawing should start (usually the same as p).
  */
  public StartDrawInfo beginDraw (Position p) {
    StartDrawInfo sdi = new StartDrawInfo ();
    setDrawInfo (sdi, p);
    sdi.first = p;
    return sdi;
  };
  
  
  /** This method is called when a new text has been
  * inserted between n and p.
  *
  * z@param n the begin of inserted text
  * @param p the end of inserted text
  * @return information about where to start redrawing (field
  * first - usually before n), where to stop redrawing (field
  * last - should be after p) and the information about how
  * to draw the first part of text
  * @warning the first and last positions should be on the same line
  * as positions n and/or p.
  * @see TextDrawInfo
  */
  public StartDrawInfo beginInsert (Position n, Position p) {
    StartDrawInfo sdi = new StartDrawInfo ();
    setDrawInfo (sdi, n);
    sdi.first = n;
    sdi.last = p;
    return sdi;
  };

  
  /** This method is called when a text s is deleted
  * from position p.
  *
  * @param p the position
  * @param s the deleted text
  * @return the information about where to start, where to
  * finish redrawing and which style to use at the
  * first part of text.
  * @warning the first and last positions should be on the same line
  * as position n.
  */
  public StartDrawInfo beginDelete (Position p, String s) {
    StartDrawInfo sdi = new StartDrawInfo ();
    setDrawInfo (sdi, p);
    sdi.first = p;
    sdi.last = p;
    return sdi;
  };

  
  /** Returns info about how the text from position p should be painted.
  * @param dis the position
  * @return the info about the text
  */
  public TextDrawInfo getDrawInfo (TextDrawInfo dis) {
    setDrawInfo (dis, dis.end);
    return dis;
  };

  /** Sets values to the draw info */
  private void setDrawInfo (TextDrawInfo di, Position p) {
    di.text = Color.white;
    di.font = f;
    di.end = p.getDocument ().getEnd ();
  };

  public Color getDefaultBackground () {
    return null;
  }

};


