/**
* Xelfi programming environment
*/

package xelfi.editor;
import java.io.IOException;
/**
* Interface, that can be passed to the Editor class and then will be called everytime
* a new document is loaded into memory.
*
* @version 1.1 This version changes the newDoc... method and adds a exception to it
* @version 1.0
* @author Jaroslav Tulach
*/

public interface EditorLoader {
  /** This method is called when a new document is loaded into memory.
  * @param s the source object of the class
  * @exception IOException is thrown when the source is not valid and the creating
  *   of it is not possible.
  */
  public void newDocumentHasBeenLoaded (Source s) throws IOException;

  /** This method is called when an old document is about to close.
  * @param s the source object of the class
  * @warning the source object s will became invalid after finishing this
  * method. So <EM>do not store pointer to it anywhere!</EM>
  * If you do not want to relase the document from memory create new
  * Source object for class s.getName ().
  * But again <EM>do not use the s object itself</EM>.
  */
  public void documentIsAboutToClose (Source s);
};


