/**
* Xelfi programming environment
*/

package xelfi.editor;
/**
* The implementation of position. They are attached to line and the line manages them (shifts them in respect of
* new inserts or deletes).
*
* @version 1.0
* @author Jaroslav Tulach
*/

class Index extends IndexBase {
  /** The previous index of the line. The first index has this field equal to null. */
  IndexBase prev;

  /** the distance from the previous position */
  short delta;

  /** the number of references to this Index */
  short used;

  /** Creates new index after this one specified. With a specified distance.
  * @param i the index
  * @param d the distance
  * @warning the distance must be less than the delta of next position, otherwise won't work
  */
  Index (IndexBase i, short d) {
    next = i.next;
    i.next = this;
    prev = i;
    delta = d;
    if (next != null) {
      next.prev = this;
      next.delta -= d;
    };
  };
  
  /** Converts integer to short. Does the same as previous one. */
  Index (IndexBase i, int d) {
    this (i, (short)d);
  };

  /** The previous index of the line.
  * @return the previous index on the line, or null if there is no such one
  */
  final Index getPrev () {
    return (prev instanceof Index) ? (Index)prev : null;
  };

  /** Gets the line that this index belongs to. Asks the previous index.
  * @return the line
  */
  final Line getLine () {
    return prev.getLine ();
  };
};

