/**
* Xelfi programming environment
*/

package xelfi.editor;
import java.io.IOException;
import java.util.StringTokenizer;
import java.util.Vector;
import java.util.Enumeration;
import java.util.NoSuchElementException;
import java.io.InputStream;
import java.io.FileInputStream;
import java.io.DataInputStream;
import java.io.FileOutputStream;
import java.io.DataOutputStream;
import java.io.File;
import java.awt.Event;
/**
* This object contains every information that can be handy for object loaded
* for a class. You can ask its class name, name of file the document is
* loaded from ...
*
* @version 1.0
* @author Jaroslav Tulach
*/

class LoadedDocument extends Document {
  /** number of references */
  int refs = 0;

  /** name of class */
  String className;

  /** name of file */
  File fileName;

  /** the xelfi file */
  XelfiFile xelfiFile;

  /** all source listeners */
  Vector list = new Vector ();
  
  /** the handler of requests for this document */
  DocumentHandler handler = StandardDocumentHandler.standardDocumentHandler;

  /** Contructor take name of class and tries to find the right source
  * file. If source does not exist then new one is created and file name
  * is set to ???. But if the class has already been compiled than there is
  * something wrong and IOException is raised.
  * @param xf the xelfi file
  * @param create create new class if it does not exist?
  * @exception IOException if source is not found but the class exists
  */
  public LoadedDocument (XelfiFile xf, boolean create) throws IOException {
    try {
      xelfiFile = xf;
      fileName = xelfiFile.getFile ();
      className = xf.getPackageAndName ();
      DataInputStream dis = new DataInputStream (new FileInputStream (fileName));
      getBegin ().insert (dis);
      dis.close ();
    } catch (NotAllowedException ex) {
      // never happens
    } catch (IOException ex) {
      if (!create) throw ex;
      // created new class
    };
    setModified (false);
  };

  /** Gets reference. Increments count of references.
  */
  synchronized void lockSource () {
    refs++;
  };

  /** Release a reference.
  * If there is no next reference then Editor remove document is called.
  * @param s the source
  */
  synchronized void unlockSource (Source s) {
    if (--refs == 0) {
      if (isModified ()) {
        // if modified then register the source to the list of modified ones
        Editor.modified.put (className, s);
      } else {
        // otherwise notify that the document is about to close
        Editor.notifyOldDocument (s);
  
        // test if a new reference to this document has not been created
        if (refs == 0) {
          // removes reference to this document
          Editor.remove (className);
        };
      };
    };
  };
};




