/**
* Xelfi programming environment
*/

package xelfi.editor;
/**
* Is responsible for maintaining relationship between line numbers
* and positions in text documents.
* When a new instance of this class is created it makes a "photo" of
* the current state of source files and remembers positions of
* all lines in source code. These informations are preserved under
* any modification of documents.
* This class offers methods thats converts sources and line to
* positions and vice versa.
*
* This class can be constructed by calling Editor.getSnapshot ()
* method.
*
* @see Editor.getSnapshot
*
* @version 1.0
* @author Jaroslav Tulach
*/

public class Snapshot extends Object {
  /** the implementation of the snapshot */
  private SnapshotImpl impl = new SnapshotImpl ();

  /** Has no public constructors.
  */
  Snapshot () {
  };
  
  /** Converts postion info to position in document.
  * @param pi the position info
  * @return the position in text document
  */
  public Position getPosition (PositionInfo pi) {
    return impl.getPosition (pi.getSource (), pi.getLine ());
  };

  /** Converts source and line index to position in document.
  * @param s the source
  * @param i the index
  * @return the position in text document
  */
  public Position getPosition (Source s, int i) {
    return impl.getPosition (s, i);
  };

  /** Converts position to position info.
  * @param p the position
  * @return the position info
  */
  public PositionInfo getPositionInfo (Position p) {
    return impl.getPositionInfo (p);
  };

  /** Finalizer. */
  public void finalize () {
    impl.close ();
  };
};


