/**
* Xelfi programming environment
*/

package xelfi.editor;
import java.util.Hashtable;
import java.util.Enumeration;
import java.util.NoSuchElementException;
/**
*
* @version 1.0
* @author Jaroslav Tulach
*/

class SnapshotImpl extends Object implements EditorLoader {
  /** maps opened documents to arrays with position of line beginings */
  Hashtable opened = new Hashtable ();

  /** Registers all opened files and puts itself to the editor loader list.
  */
  SnapshotImpl () {
    synchronized (this) {
      Editor.addLoader (this);
      
      Enumeration en = Editor.loadedSources ();
      try {
        for (;;) {
          newDocumentHasBeenLoaded ((Source)en.nextElement ());
        }
      } catch (NoSuchElementException ex) {
      }
    }
  }

  /** Closes itself.
  */
  synchronized void close () {
    Editor.removeLoader (this);
  }

  /** This method is called when a new document is loaded into memory.
  * Adds this source to hashtable of opened files.
  * @param s the source object of the class
  */
  public synchronized void newDocumentHasBeenLoaded (Source s) {
    Object[] dd = (Object[])opened.get (s.getName ());
    if (dd != null) {
      // the source has been previously loaded
      // the data is object of class Integer[]
      Integer[] array = (Integer[])dd;
      Document d = s.getDocument ();
      Position[] data = new Position[array.length];
      
      for (int i = 0; i < array.length; i++) {
        try {
          data[i] = d.goToLine (array[i].intValue ());
        } catch (EndOfTextException ex) {
          data[i] = d.getBegin ();
        }
      }
      opened.put (s.getName (), data);
    } else {
      // the document has not been modified yet
      int lines = s.getDocument ().numberOfLines ();
      Position[] array = new Position[lines + 1];
      Position p = s.getDocument ().getBegin ();
      opened.put (s.getName (), array);
      try {
        for (int i = 0;;i++, p = p.down ()) {
          array[i] = p;
        }
      } catch (EndOfTextException ex) {
      } catch (ArrayIndexOutOfBoundsException ex) {
      }
    }
  }

  /** This method is called when an old document is about to close.
  * @param s the source object of the class
  * @warning the source object s will became invalid after finishing this
  * method. So <EM>do not store pointer to it anywhere!</EM>
  * If you do not want to relase the document from memory create new
  * Source object for class s.getName ().
  * But again <EM>do not use the s object itself</EM>.
  *
  * <p>
  * In this implementation does nothing.
  */
  public synchronized void documentIsAboutToClose (Source s) {
    // we must convert array of position to array of int
    Position[] array = (Position[])opened.get (s.getName ());
    // the data is object of class Integer[]
    Document d = s.getDocument ();
    Integer[] data = new Integer[array.length];
      
    for (int i = 0; i < array.length; i++) {
      try {
        data[i] = new Integer (d.lineToInt (array[i]));
      } catch (InvalidPositionException ex) {
        data[i] = new Integer (0);
      }
    }
    opened.put (s.getName (), data);
  }


  /** Converts postion info to position in document.
  * @param s the source
  * @param l the line
  * @return the line
  */
  public synchronized Position getPosition (Source s, int l) {
    if (l < 0) return s.getDocument ().getBegin ();
    Position[] array = (Position[])opened.get (s.getName ());
    if (l >= array.length) return s.getDocument ().getEnd ();
    return array[l];
  }

  /** Converts position to position info.
  * @param p the position
  * @return the position info
  */
  public synchronized PositionInfo getPositionInfo (Position p) {
    p = p.beginOfLine ();
    Source s = new Source ((LoadedDocument)p.getDocument ());
    // at last now the document is loaded in memory
    
    Position[] array = (Position[])opened.get (s.getName ());
    
    int from = 0;
    int end = array.length;
    int indx = 1;

    while (from < end) {
      indx = from + (end - from) / 2;
      if (array[indx].sameLine (p)) break;
      if (array[indx].lessOrEqual (p)) {
        from = indx + 1;
      } else {
        end = indx;
      }
    }
    return new PositionInfo (s, indx);
  }
}


