/**
* Xelfi Environment
*/

package xelfi.editor;
import java.util.Hashtable;
import java.awt.Window;

/**
*
* @author Jaroslav Tulach
* @version 1.0
*/
public interface TextEditorInfo {
  /** Sets new editor callback to the editor. The previous one
  * is returned.
  * @param c the new callback
  */
  public void setEditorCallback (EditorCallback c);
  
  /** Only returns editor callback.
  * @return the editor callback
  */
  public EditorCallback getEditorCallback ();
  
  /** Gets the document in editor window.
  * @return the document
  */
  public Document getDocument ();

  /** Gets the current position in document
  * @return the position
  */
  public Position getCursorPos ();

  /** Sets the cursor. If it is out of screen then changes the screen view.
  * @param pos the position to set the cursor to
  */
  public void setCursorPos (Position pos);

  /** Scrolls up one line. Moves everything on the screen down and draws
  * the first line.
  * @exception EndOfTextException if begin of document reached
  */
  public void scrollUp () throws EndOfTextException;
  
  /** Scrolls down one line. Moves everything on the screen up and draws
  * the bottom line.
  * @exception EndOfTextException if end of document reached
  */
  public void scrollDown () throws EndOfTextException;
  
  /** Gets currently selected block.
  * @return the block
  */
  public Block getSelectedBlock ();

  /** Sets selected block to new value.
  * @param sel the block or null to clear the selection
  */
  public void setSelectedBlock (Block sel);

  /** Gets visible block
  * @return the block that is visible in the editor
  */
  public Block getVisibleBlock ();

  /** Sets visible block begin.
  * @param p position where the visible part should begin
  */
  public void setVisibleBlockBegin (Position p);

  /** Get number of visible lines.
  * @return the number of lines that fits into editor window
  */
  public int numberOfVisibleLines ();

  /** Begins text marking. This call should be followed by a call
  * to blockEdxtendEnd.
  */
  public void blockExtendBegin ();

  /** Ends marking. Must be called after startMark. Changes selected block.
  *
  */
  public void blockExtendEnd ();

  /** Returns editor hashtable. This should be used to store
  * information about the editor like special position,
  * find and replace dialog and so on...
  * <p>
  * Keys to the table should be strings and should start with the name
  * of class that operates on the hash table followed with @ and
  * identificator. Like:
  * <P>
  * <center>xelfi.editor.Editor@Position1</center>
  * @return the hash table
  */
  public Hashtable getHashtable ();

  /** Gets frame or dialog where this editor is stored.
  * @return the window where this editor is stored
  */
  public Window getWindow ();


  /** Adds new cursor listener.
  * @param l the listener
  * @see CursorListener
  */
  public void addCursorListener (CursorListener l);
  
  /** Adds new editor listener.
  * @param l the listener
  * @see EditorListener
  */
  public void addEditorListener (EditorListener l);
  
  /** Removes listener.
  * @param l the listener
  * @return true if l was in the listener list
  * @see CursorListener
  */
  public boolean removeCursorListener (CursorListener l);
  
  /** Removes listener.
  * @param l the listener
  * @return true if l was in the listener list
  * @see EditorListener
  */
  public boolean removeEditorListener (EditorListener l);
}










// Generated from substitution template 'Interface' Blocks:Class:17,11,31,11; Package:8,4,20,4; 
