/**
* Xelfi Environment
*/

package xelfi.editor.keys;
import xelfi.editor.*;
import java.util.*;

/** Executes an action on the key.
*
* @author Jaroslav Tulach
* @version 1.0
*/
class ContextAction extends Object implements ContextItem {
  /** handler's vector */
  KeyActionMacro handlers;

  /**
  * @param handler the handler to execute on this action
  */
  public ContextAction (KeyActionMacro v) {
    this.handlers = v;
  }

  /** Accepts the key by calling the handler.
  * @param key the key
  * @param info editor info
  * @return null to indicate that this key sequence has ended
  * @exception NotAllowedException if the action is not allowed
  * @exception EndOfTextException if end of text is reached
  */
  public ContextTable acceptKey (KeyCode key, TextEditorInfo info)
  throws NotAllowedException, EndOfTextException {
    handlers.keyDown (key.key, key.mod, info);
    return KeyOperator.basic;
  };

  /** Does not add the key. Indicates error.
  * @param key the key
  * @param commands the commands to excute
  * @return false to indicate error.
  */
  public boolean addKey (KeyCode key, KeyActionMacro commands) {
    return false;
  };

  /** toString.
  * @return string
  */
  public String toString () {
    StringBuffer sb = new StringBuffer ("Action ");
    Enumeration en = handlers.elements ();
    while (en.hasMoreElements ()) {
      KeyActionHandler k = (KeyActionHandler) en.nextElement ();
      sb.append (k.getClass ().getName ());
      sb.append (' ');
    };
    return sb.toString ();
  };
}

// Generated from substitution template 'Class' Blocks:Class:9,20,22,20; Class:6,13,19,13; Package:8,4,25,4; 
