/**
* Xelfi programming environment
*/

package xelfi.xedit;
import java.io.FileInputStream;
import java.io.File;
/**
* Handles inicialization of all objects in editor, xedit and templates
* packages.
*
* @version 1.0
* @author Jaroslav Tulach
*/

public abstract class XelfiEditor extends Object {
  /** Basic templates' strings.
  */
  private static String[] templates = {
    "Package#xelfi.xedit.SubstManager;Package:8,4,11,4;",
    "Class#xelfi.xedit.SubstManager;Package:8,4,11,4; Class:13,11,16,11; Class:9,15,12,15;",
    "Exception#xelfi.xedit.SubstManager;Package:8,4,11,4; Class:13,11,16,11; Class:9,15,12,15; Class:28,13,31,13; Class:28,19,31,19; Class:9,23,12,23;",
    "Interface#xelfi.xedit.SubstManager;Package:8,4,11,4; Class:17,11,20,11;",
    "Main:Class#xelfi.xedit.SubstManager;Package:8,4,11,4; Class:13,11,16,11; Class:9,15,12,15;",
    "Application#xelfi.design.forms.ApplicationManager",
    "Frame#xelfi.design.forms.FrameManager",
    "Dialog#xelfi.design.forms.DialogManager",
    "Applet#xelfi.design.forms.AppletManager"
  };

  /** Initialization of classes that need to be inicialized in packages
  * xelfi.editor, xelfi.xedit and xelfi.templates.
  *
  * This method should be called by top during the inicialization
  * phase.
  */
  public static void initialize () {
    String path = xelfi.util.XelfiPath.get ();
    path += File.separatorChar + "editor" + File.separatorChar +
      "actions" + File.separatorChar + "Default.key";
    try {
      xelfi.util.Debug.debug ("Reading keys from " + path);
      FileInputStream is = new FileInputStream (path);
      xelfi.editor.keys.KeyOperator.load (is);
      is.close ();
      
    } catch (java.io.FileNotFoundException ex) {
      System.err.println ("Cannot open file: " + path);
      System.exit (1);
    } catch (xelfi.editor.keys.EditorKeyException ex) {
      System.err.println ("Parse error: " + ex.getMessage ());
      System.exit (2);
    } catch (java.io.IOException ex) {
    };
    
    xelfi.editor.Editor.addLoader (new SourceSaver ());
    xelfi.top.TopLevel.addOption (new TemplateOptions ());
    xelfi.top.TopLevel.addOption (new EditorOptions ());
    xelfi.top.XFColor.addCallback (new XelfiEditorColorCallback ());
    xelfi.templates.Template.initialize ();

    for (int i = 0; i < templates.length; i++) {
      try {
        xelfi.templates.Template.fromString (templates[i]);
      } catch (Exception ex) {
        System.err.println ("Cannot create basic template: " + templates[i]);
        System.err.println (ex);
      };
    };
  };
};










